package com.yvan.platform;

import com.google.common.collect.Lists;
import lombok.SneakyThrows;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

public class ReflectUtil {
    /***
     * 获取私有成员变量的值
     */
    @SneakyThrows
    public static Object getValue(Object instance, String fieldName) {

        Field field = instance.getClass().getDeclaredField(fieldName);
        field.setAccessible(true);

        return field.get(instance);
    }

    /**
     * 调用私有成员方法
     */
    @SneakyThrows
    public static Object invoke(Object obj, String methodName, Object... params) {
        int paramCount = params.length;
        Method method;
        Class<?>[] classArray = new Class<?>[paramCount];
        for (int i = 0; i < paramCount; i++) {
            classArray[i] = params[i].getClass();
        }
        method = obj.getClass().getDeclaredMethod(methodName, classArray);
        method.setAccessible(true);
        return method.invoke(obj, params);
    }

    /**
     * 为私有成员变量赋值
     */
    @SneakyThrows
    public static void setValue(Object instance, String fieldName, Object value) {
        Field field = instance.getClass().getDeclaredField(fieldName);
        field.setAccessible(true);

        field.set(instance, value);
    }

    /**
     * 为私有成员变量赋值
     */
    @SneakyThrows
    public static void setValue(Class<?> clazz, Object instance, String fieldName, Object value) {
        Field field = clazz.getDeclaredField(fieldName);
        field.setAccessible(true);

        field.set(instance, value);
    }

    /**
     * 利用反射实现对象之间相同属性复制
     */
    public static void copyProperties(Object source, Object target) throws Exception {

        copyPropertiesExclude(source, target, null);
    }

    /**
     * 复制对象属性
     *
     * @param from
     * @param to
     * @param excludsArray 排除属性列表
     * @throws Exception
     */
    public static void copyPropertiesExclude(Object from, Object to,
                                             String[] excludsArray) throws Exception {
        List<String> excludesList = null;

        if (excludsArray != null && excludsArray.length > 0) {
            excludesList = Lists.newArrayList(excludsArray);
        }

        Method[] fromMethods = from.getClass().getDeclaredMethods();
        Method[] toMethods = to.getClass().getDeclaredMethods();
        Method fromMethod = null, toMethod = null;
        String fromMethodName = null, toMethodName = null;

        for (int i = 0; i < fromMethods.length; i++) {

            fromMethod = fromMethods[i];
            fromMethodName = fromMethod.getName();

            if (!fromMethodName.contains("get"))
                continue;
            // 排除列表检测
            if (excludesList != null
                    && excludesList.contains(fromMethodName.substring(3)
                    .toLowerCase())) {

                continue;
            }
            toMethodName = "set" + fromMethodName.substring(3);
            toMethod = findMethodByName(toMethods, toMethodName);

            if (toMethod == null)
                continue;
            Object value = fromMethod.invoke(from, new Object[0]);

            if (value == null)
                continue;
            // 集合类判空处理
            if (value instanceof Collection) {
                Collection<?> newValue = (Collection<?>) value;
                if (newValue.size() <= 0)
                    continue;
            }

            toMethod.invoke(to, new Object[]{value});
        }
    }

    /**
     * 对象属性值复制，仅复制指定名称的属性值
     */
    public static void copyPropertiesInclude(Object from, Object to,
                                             String[] includsArray) throws Exception {

        List<String> includesList = null;

        if (includsArray != null && includsArray.length > 0) {

            includesList = Arrays.asList(includsArray);

        } else {

            return;
        }
        Method[] fromMethods = from.getClass().getDeclaredMethods();
        Method[] toMethods = to.getClass().getDeclaredMethods();
        Method fromMethod = null, toMethod = null;
        String fromMethodName = null, toMethodName = null;

        for (int i = 0; i < fromMethods.length; i++) {

            fromMethod = fromMethods[i];
            fromMethodName = fromMethod.getName();

            if (!fromMethodName.contains("get"))
                continue;

            // 排除列表检测
            String str = fromMethodName.substring(3);

            if (!includesList.contains(str.substring(0, 1).toLowerCase()
                    + str.substring(1))) {
                continue;
            }

            toMethodName = "set" + fromMethodName.substring(3);
            toMethod = findMethodByName(toMethods, toMethodName);

            if (toMethod == null)
                continue;

            Object value = fromMethod.invoke(from, new Object[0]);

            if (value == null)
                continue;

            // 集合类判空处理
            if (value instanceof Collection) {

                Collection<?> newValue = (Collection<?>) value;

                if (newValue.size() <= 0)
                    continue;
            }

            toMethod.invoke(to, new Object[]{value});
        }
    }


    /**
     * 从方法数组中获取指定名称的方法
     */
    public static Method findMethodByName(Method[] methods, String name) {
        for (int j = 0; j < methods.length; j++) {
            if (methods[j].getName().equals(name)) {
                return methods[j];
            }
        }
        return null;
    }
}
